package dev.ninjdai.werewolf.uhc.roles;

import dev.ninjdai.werewolf.uhc.effects.Effect;
import lombok.Getter;
import net.minestom.server.item.ItemStack;
import net.minestom.server.tag.Tag;

import java.util.*;

public class Role {
    public Role(String id, String name, List<ItemStack> roleItems, List<Effect> roleEffects, Set<RoleProperties> roleProperties) {
        this.id = id;
        this.name = name;
        this.roleItems = roleItems;
        this.roleEffects = roleEffects;
        this.roleProperties = roleProperties;

        roleMap.put(this.id, this);
    }

    Role(Builder builder) {
        this.id = builder.id;
        this.name = builder.name;
        this.roleItems = builder.roleItems;
        this.roleEffects = builder.roleEffects;
        this.roleProperties = builder.roleProperties;

        roleMap.put(this.id, this);
    }

    @Getter private String id;
    @Getter private String name;
    @Getter private List<ItemStack> roleItems;
    @Getter private List<Effect> roleEffects;
    @Getter private Set<RoleProperties> roleProperties;

    public static Tag<String> roleTag = Tag.String("role");

    // ID: Role
    public static Map<String, Role> roleMap = new HashMap<>();

    public static class Builder {
        String id = "dummy";
        String name = "Dummy";
        private List<ItemStack> roleItems = List.of();
        private List<Effect> roleEffects = List.of();
        Set<RoleProperties> roleProperties = new HashSet<>();

        public Builder setId(String id) {
            this.id = id;
            return this;
        }

        public Builder setName(String name) {
            this.name = name;
            return this;
        }

        public Builder setRoleItems(List<ItemStack> roleItems) {
            this.roleItems = roleItems;
            return this;
        }

        public Builder addRoleItems(ItemStack ... roleItems) {
            this.roleItems.addAll(Arrays.asList(roleItems));
            return this;
        }

        public Builder setRoleEffects(List<Effect> roleEffects) {
            this.roleEffects = roleEffects;
            return this;
        }

        public Builder addRoleEffects(Effect... roleEffects) {
            this.roleEffects.addAll(Arrays.asList(roleEffects));
            return this;
        }

        public Builder setRoleProperties(Set<RoleProperties> roleProperties) {
            this.roleProperties = roleProperties;
            return this;
        }

        public Builder addRoleProperties(RoleProperties ... roleProperties) {
            this.roleProperties.addAll(List.of(roleProperties));
            return this;
        }

        public Role build() {
            return new Role(this);
        }
    }
}
